/*
 * RFM12B.c
 *
 *  Created on: 19-02-2014
 *      Author: Piotr Rzeszut
 *  
 * Description: Funkcje do obsugi ukadu RFM12B z poziomu procesora AVR
 */
#include "RFM12B.h"

void Rfm_spi_init(void){
	SPI_DDR |= (1<<SPI_SCK)|(1<<SPI_MOSI)|(1<<SPI_SS);//konfiguracja kierunku i podcigania linii SPI
	SPI_PORT |= (1<<SPI_MISO);
	//do poprawnej pracy moduu SPI w procesorze linia SS musi by ustawiona jako wyjcie

	SPCR |= (1<<SPE)|(1<<MSTR)|(1<<SPR0);//SPI w trybie master z podziaem zegara przez 64 
	

	CS_PORT |= (1<<CS_RFM);//dezaktywacja ukadu RFM12B
	CS_DDR |= (1<<CS_RFM);//konfiguracja kierunku linii CS
}

uint16_t Rfm_xmit(uint16_t data){
	//na pocztek rodzielamy dane na 2 bajty
	uint8_t msb, lsb;
	lsb = data;
	msb = data>>8;
	CS_PORT &= ~(1<<CS_RFM);//aktywujemy lini CS ukadu
	//teraz wysyamy 2 bajty jednoczenie odbierajc 2 bajty z ukadu
	SPDR = msb;
	while(!(SPSR&(1<<7)));
	msb = SPDR;
	SPDR = lsb;
	while(!(SPSR&(1<<7)));
	lsb = SPDR;
	//_delay_ms(1);
	CS_PORT |= (1<<CS_RFM);//dezaktywujemy lini CS ukadu
	//i zwracamy odebrane dane
	return( (((uint16_t)msb)<<8)+((uint16_t)lsb) );
}

void Rfm_init(void){
	Rfm_xmit(SW_RESET);//resetuj programowo ukad RFM12B
	_delay_ms(250);

	//inicjalizacja RFM12B
	//ustawienie pasma 868MHz, konfiguracja FIFO
	Rfm_xmit(CONFIGURATION|EN_DATA_REG|EN_FIFO|BAND_868|CAP_12_0);
	//wczenie oscylatora
	Rfm_xmit(POWER|EN_OSC|DIS_CLKO);
	//ustawienie pasma (musi by takie samo w nadajniku i odbiorniku)
	//Dla naszego ukadu czstotliwo musi zawiera si w przedziale 860480000-879515000Hz i musi by podawana ze skokami 5000Hz
	Rfm_xmit(FREQUENCY|RF12_FREQUENCY_CALC_868(868000000UL));
	//ustawienie prdkoci transmisji danych (musi by takia sama w nadajniku i odbiorniku)
	Rfm_xmit(BAUD|BAUD_4800);
	//ustawienie pasma 134kHz i parametrw odbiornika
	Rfm_xmit(RECEIVER|P20_VDI|BW134|LNA_0|RSSI_103);
	//ustawienie cyfroiwego filtra danych i opcji odzyskiwania zegara
	Rfm_xmit(DATA_FILTER|AUTO_CR|DIGITAL_F|DQD_4);
	//reset bufora FIFO, konfiguracja synchronizacji za pomoc 2 bajtw, ustawienie generowania przerwania FFIT po odebraniu 8 bitw
	Rfm_xmit(FIFO_RST|FIFO_IT_8|FIFO_SYNC|HS_RST_DIS);
	

	//konfiguracja kontrolera czstotliwoci
	Rfm_xmit(AFC|KEEP_RX|REST_OFF|EN_FOFFSET|EN_AFC);
	//konfiguracja nadajnika i jego mocy (na ustawienie maksymalne)
	Rfm_xmit(TRANSMITER|FSK_PHASE_0|FSK_DF_90K|OUT_P_0);
	//konfiguracja ptli PLL
	Rfm_xmit(PLL|PLL_DH_DIS|SEL_CLK_2_5|MAX_BAUD_256);

	//wyczenie timera wakeup
	Rfm_xmit(WAKEUP_TIM|WUT_X_2_0|0);
	//wyczenie oszczdzania energii
	Rfm_xmit(LOW_DC|LOW_DC_DIS);
	//ustawienie monitora napicia na 2,2V
	Rfm_xmit(BOD_CLK|CLK_5_00|BOD_2_2);
}

uint8_t Rfm_ready_wait(void){
	uint8_t i=0;
	
	CS_PORT &= ~(1<<CS_RFM);//zaczam pin CS ukadu
	_delay_us(1);//czekam nieco aby ukad mg zareagowa
	
	while(!((1<<SPI_MISO)&SPI_PIN)){//nastpnie co 1ms sprawdzam, czy ukad jest wolny, tzn, czy wystawi na linii MISO 1.
		_delay_ms(1);
		if((++i)==200){
			return 1;//po ok. 200ms przerywam oczekiwanie i zwracam 1 - czyli sygnalizuj bd
		}
	}
	return 0;//jeli ptal zostanie przerwana z powodu wystpienia stanu wysokiego na linii MISO to zwracam 0 - nie ma bdu
}
