/*
 * main.c
 *
 *  Created on: 19-02-2014
 *      Author: Piotr Rzeszut
 *  
 * Description: Przykad komunikacji z moduem RFM12B
 */
#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>
#include <avr/interrupt.h>

#include "RFM12B.h"
#include "uart.h"

char bufor[65];

//jeli ponisza linia bdzie zakomentowana to skompilujemy kod dla ukadu SLAVE,
//natomiast jeli bdzie odkomentowana to skompilujemy kod dla ukadu SLAVE
//#define MASTER

#ifdef MASTER
//--------------------------------------------------------------------------------------------------------
//Kod dla ukadu MASTER
//--------------------------------------------------------------------------------------------------------
int main(void){

	Rfm_spi_init();//inicjalizacja magistrali SPI
	uart_init(9600);//inicjalizacja USART
	sei();//wczamy przerwania do obsugi uart

	uart_puts("\r\n\r\nRFM12B - RECEIVER\r\n");//wywietlamy powitanie
	
	Rfm_init();//inicjalizujemy ukad RFM12B
	
	Rfm_xmit(SYNC_PATTERN|0xD4);
	//ustawiamy programowalny bajt synchronizacji na warto 0xD4
	//wykorzystamy t funkcjonalno do adresowania wielu ukadw
	
	//wczamy odbiornik
	Rfm_xmit(POWER|EN_RX|EN_BASEBAND|EN_OSC|DIS_CLKO);
	_delay_ms(5);
	
	while(1){
		//para komend powodujca w efekcie dziaania reset synchronizacji odbiornika
		Rfm_xmit(FIFO_RST|FIFO_IT_8|HS_RST_DIS);
		Rfm_xmit(FIFO_RST|FIFO_IT_8|EN_AFT_SYNC|HS_RST_DIS);
		
		uint8_t timeout=Rfm_ready_wait();//oczekujemy na odebranie przez ukad danych
		if(timeout){//i w zalenoci od tego czy ukad odpowiedzia
			//albo wywietlamy informacje o braku odebranych danych
			uart_puts("NO INCOMING DATA\r\n");
		}else{
			//albo odbieramy dane z bufora
			uint8_t data=Rfm_xmit(FIFO_READ);
			//i wywietlamy je
			sprintf(bufor,"INCOMING DATA:%d\r\n",data);
			uart_puts(bufor);
		}
	}

	return 0;
}

#else
//--------------------------------------------------------------------------------------------------------
//Kod dla ukadu SLAVE
//--------------------------------------------------------------------------------------------------------
int main(void){

	Rfm_spi_init();//inicjalizacja magistrali SPI
	
	Rfm_init();//wstpna konfiguracja ukadu RFM12B
	
	DDRB &= ~((1<<PB0)|(1<<PB1));//konfiguruj piny PB0-1 jako wejcie
	PORTB |= (1<<PB0)|(1<<PB1);//i wczam podciganie
	
	uint8_t data;
	
	while(1){
		//sprawdzam przyciski i ustawiam odpowiednie bity w bajcie do nadania
		data = 0;
		if(!(PINB&(1<<PB0))){
			data |= 1;
		}
		if(!(PINB&(1<<PB1))){
			data |= 2;
		}
		if(data==0)continue;//jeli nic nie wcinito to rozpoczynamy ptl od pocztku
		
		//wczamy nadajnik
		Rfm_xmit(POWER|EN_TRANSMISSION|EN_SYNTH|EN_OSC|DIS_CLKO);
		//nadajemy 3 bajty synchronizacji (0xAA)
		//dwa pierwsze s ju domylnie umieszczone w buforze nadawczym po komendzie wczajcej nadajnik
		Rfm_ready_wait();
		Rfm_xmit(TX_WRITE|0xAA);
		//nastpnie kolejny bajt synchrnoizacji (0x2D)
		Rfm_ready_wait();
		Rfm_xmit(TX_WRITE|0x2D);
		Rfm_ready_wait();
		//oraz definiowalny bajt synchronizacji (czyli nasz adres)
		Rfm_xmit(TX_WRITE|0xD4);
		Rfm_ready_wait();
		Rfm_xmit(TX_WRITE|data);
		Rfm_ready_wait();
		//oraz pusty bajt - konieczny by poprawnie zakoczy trnasmisj
		Rfm_xmit(TX_WRITE|0xAA);
		Rfm_ready_wait();
		//nastpnie czekamy na zakoczenie transmisji
		uint16_t status;
		do 
		{
			status=Rfm_xmit(STATUS_READ);
		} while (!(status&M_TX_READY));
		//i wyczamy nadajnik
		Rfm_xmit(POWER|DIS_CLKO);
		_delay_ms(100);
	}

	return 0;
}

#endif
